# Generated by Django 4.2 on 2023-04-22 15:36

from django.conf import settings
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0012_alter_user_first_name_max_length'),
        ('contenttypes', '0002_remove_content_type_name'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('safety', '0007_alter_objectpermission_options'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='PermissionGroup',
            new_name='ObjectGroup',
        ),
    ]
