# Generated by Django 4.2 on 2023-04-18 23:07

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('auth', '0012_alter_user_first_name_max_length'),
        ('safety', '0004_alter_objectpermission_unique_together_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='permissiongroup',
            name='target_id',
            field=models.IntegerField(null=True, verbose_name='Target ID'),
        ),
        migrations.AlterUniqueTogether(
            name='objectpermission',
            unique_together={('to_ct', 'to_id', 'permission', 'object_ct', 'object_id')},
        ),
    ]
