# Generated by Django 4.2 on 2023-04-22 15:06

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('safety', '0005_alter_permissiongroup_target_id_and_more'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='objectpermission',
            options={'verbose_name': 'User object permission', 'verbose_name_plural': 'User object permissions'},
        ),
    ]
