# Generated by Django 4.2 on 2023-04-16 21:27

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('auth', '0012_alter_user_first_name_max_length'),
        ('contenttypes', '0002_remove_content_type_name'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('safety', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='objectpermission',
            name='object_ct',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype',
                                    verbose_name='Target Content Type'),
        ),
        migrations.AlterField(
            model_name='objectpermission',
            name='object_id',
            field=models.IntegerField(verbose_name='Object ID'),
        ),
    ]
