# Generated by Django 4.2 on 2023-04-22 15:19

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('safety', '0006_alter_objectpermission_options'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='objectpermission',
            options={'verbose_name': 'User Object Permission', 'verbose_name_plural': 'User Object Permissions'},
        ),
    ]
